// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

package otel // import "go.opentelemetry.io/otel"

// Version is the current release version of OpenTelemetry in use.
func Version() string {
	return "1.43.0"
}
