// Code generated by "stringer -type=Temporality"; DO NOT EDIT.

package metricdata

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[undefinedTemporality-0]
	_ = x[CumulativeTemporality-1]
	_ = x[DeltaTemporality-2]
}

const _Temporality_name = "undefinedTemporalityCumulativeTemporalityDeltaTemporality"

var _Temporality_index = [...]uint8{0, 20, 41, 57}

func (i Temporality) String() string {
	idx := int(i) - 0
	if i < 0 || idx >= len(_Temporality_index)-1 {
		return "Temporality(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _Temporality_name[_Temporality_index[idx]:_Temporality_index[idx+1]]
}
