// Copyright 2024 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package genid

const (
	NoUnkeyedLiteral_goname  = "noUnkeyedLiteral"
	NoUnkeyedLiteralA_goname = "XXX_NoUnkeyedLiteral"

	BuilderSuffix_goname = "_builder"
)
