//go:build arm64 && freebsd
// +build arm64,freebsd

// Code generated by cmd/cgo -godefs; DO NOT EDIT.
// cgo -godefs types_freebsd.go

package pty

const (
	_C_SPECNAMELEN = 0xff
)

type fiodgnameArg struct {
	Len int32
	Buf *byte
}
