// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// ConfigReference The config-only network source to provide the configuration for
// this network.
//
// swagger:model ConfigReference
type ConfigReference struct {

	// The name of the config-only network that provides the network's
	// configuration. The specified network must be an existing config-only
	// network. Only network names are allowed, not network IDs.
	//
	// Example: config_only_network_01
	Network string `json:"Network"`
}
