// Code generated by go-swagger; DO NOT EDIT.

package swarm

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// ServiceUpdateResponse service update response
// Example: {"Warnings":["unable to pin image doesnotexist:latest to digest: image library/doesnotexist:latest not found"]}
//
// swagger:model ServiceUpdateResponse
type ServiceUpdateResponse struct {

	// Optional warning messages
	Warnings []string `json:"Warnings"`
}
